// JournalRecorder.cpp: implementation of the CJournalRecorder class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "JournalHook.h"
#include "JournalRecorder.h"
#include "JournalHookDlg.h"

#include <time.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CJournalRecorder theJournalRecorder;

CJournalRecorder * AfxGetJournalRecorder() { return &theJournalRecorder; }

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CJournalRecorder::CJournalRecorder()
{
	m_fIsRunning = FALSE;
	m_nRunningIndex = 0;
	m_nScenarioState = STOP;

	m_fShakeMouse = FALSE;
	m_nShakeRange = 5;

	m_fPlayEveryday = FALSE;
	m_tsEverydayAdjustTime = CTimeSpan(0, 0, 0, 0);

	m_fInitCalled = FALSE;			// Has the Init function been called ?
	m_dwStartRecordTime = 0;		// Time JournalRecord Started

	m_lpEventChain	= NULL;  // Head of recorded Event List
	m_lpLastEvent	= NULL;  // Tail of recorded Event List
	m_lpPlayEvent	= NULL;  // Current Event being played

	//  My Hook States
	m_nHookStates[0] = 0; // State Table of my hooks
	m_nHookStates[1] = 0; // State Table of my hooks

	//  Hook Codes
	m_nHookCodes[0] = WH_JOURNALPLAYBACK;
	m_nHookCodes[1] = WH_JOURNALRECORD;

	// Filter Function Addresses
	m_lpfnHookProcs[0] = (FARPROC)JournalPlaybackFunc;
	m_lpfnHookProcs[1] = (FARPROC)JournalRecordFunc;

	m_fScreenSaverWasActived = FALSE;
	SystemParametersInfo(SPI_GETSCREENSAVEACTIVE, 0, &m_fScreenSaverWasActived, SPIF_SENDWININICHANGE);

	srand( (unsigned)time( NULL ) );
}

CJournalRecorder::~CJournalRecorder()
{
	m_pCallBack = NULL;
	EraseJournal();
	InstallFilter(JOURNAL_PLAYBACK_INDEX, FALSE);
	InstallFilter(JOURNAL_RECORD_INDEX, FALSE);
}

BOOL CJournalRecorder::Init(void (* pCallBack)(int, LPARAM) /*= NULL*/)
{
	m_fInitCalled = TRUE;
	m_pCallBack = pCallBack;

	return TRUE;
}

BOOL CJournalRecorder::RunScenario(CByteArray & anItemSelected)
{
	m_fIsRunning = TRUE;
	m_nRunningIndex = -1;

	m_fUseItemSelected = TRUE;
	m_anItemSelected.RemoveAll();
	m_anItemSelected.SetSize(anItemSelected.GetSize());
	for(int i = 0; i < anItemSelected.GetSize(); i++)
		m_anItemSelected[i] = anItemSelected[i];

	while(!m_anItemSelected[++m_nRunningIndex]);

	m_tsEverydayAdjustTime = CTimeSpan(0, 0, 0, 0);

	if(m_fPlayEveryday) {
		for(int iItem = 0; iItem < GetSize(); iItem++) {
			if(GetAt(iItem).m_fCheckStart) {
				m_tsEverydayAdjustTime = CTimeSpan((CTime::GetCurrentTime() - GetAt(iItem).m_tStartTime).GetDays(), 0, 0, 0);
				break;
			}
			if(GetAt(iItem).m_fCheckStop) {
				m_tsEverydayAdjustTime = CTimeSpan((CTime::GetCurrentTime() - GetAt(iItem).m_tStopTime).GetDays(), 0, 0, 0);
				break;
			}
		}

	}

	if(m_pCallBack)
		m_pCallBack(JOURNAL_MESSAGE, (LPARAM)"Beginning playback...");

	m_nScenarioState = IDLE;
	NextScenarioState();

	return TRUE;
}

BOOL CJournalRecorder::RunScenario()
{
	m_fIsRunning = TRUE;
	m_nRunningIndex = 0;
	m_fUseItemSelected = FALSE;

	m_tsEverydayAdjustTime = CTimeSpan(0, 0, 0, 0);

	if(m_fPlayEveryday) {
		for(int iItem = 0; iItem < GetSize(); iItem++) {
			if(GetAt(iItem).m_fCheckStart) {
				m_tsEverydayAdjustTime = CTimeSpan((CTime::GetCurrentTime() - GetAt(iItem).m_tStartTime).GetDays(), 0, 0, 0);
				break;
			}
			if(GetAt(iItem).m_fCheckStop) {
				m_tsEverydayAdjustTime = CTimeSpan((CTime::GetCurrentTime() - GetAt(iItem).m_tStopTime).GetDays(), 0, 0, 0);
				break;
			}
		}

	}

	if(m_pCallBack)
		m_pCallBack(JOURNAL_MESSAGE, (LPARAM)"Beginning playback...");

	m_nScenarioState = IDLE;
	NextScenarioState();

	return TRUE;
}

//typedef enum EScenarioState { IDLE = 0, WAIT_START, PLAY_START, PLAY_NEXT, PLAY_STOP, WAIT_STOP, STOP};

// Voici le coeur du systeme : The State Machine
// Trop complique
BOOL CJournalRecorder::NextScenarioState()
{
	switch(m_nScenarioState) {
	case IDLE:
		if(m_pCallBack)
			m_pCallBack(JOURNAL_MESSAGE, (LPARAM)"Execute State : IDLE");
		TRACE("Execute State : IDLE\n");

		if(	GetAt(m_nRunningIndex).m_fCheckStop &&
			CTime::GetCurrentTime() > GetAt(m_nRunningIndex).m_tStopTime + m_tsEverydayAdjustTime) {
			m_nScenarioState = STOP;
			NextScenarioState();

		} else if(	GetAt(m_nRunningIndex).m_fCheckStart &&
			CTime::GetCurrentTime() < GetAt(m_nRunningIndex).m_tStartTime + m_tsEverydayAdjustTime) {
			m_uMainTimerID = SetTimer(NULL, NULL,
				1000 * (GetAt(m_nRunningIndex).m_tStartTime + m_tsEverydayAdjustTime - CTime::GetCurrentTime()).GetTotalSeconds(),
				CJournalRecorder::StartTimerProc);
			TRACE("\tWait %d seconds\n", (GetAt(m_nRunningIndex).m_tStartTime + m_tsEverydayAdjustTime - CTime::GetCurrentTime()).GetTotalSeconds());
		} else {
			m_nScenarioState = PLAY_START;
			NextScenarioState();
		}
		break;
	case WAIT_START:
		if(m_pCallBack)
			m_pCallBack(JOURNAL_MESSAGE, (LPARAM)"Execute State : WAIT_START");
		TRACE("Execute State : WAIT_START\n");

		if(GetAt(m_nRunningIndex).m_tStartTime + m_tsEverydayAdjustTime <= CTime::GetCurrentTime()) {
			m_nScenarioState = PLAY_START;
			NextScenarioState();
		} else
			m_uMainTimerID = SetTimer(NULL, NULL, 1000, CJournalRecorder::StartTimerProc);

		break;
	case PLAY_START:
		TRACE("Execute State : PLAY_START\n");
		if(m_pCallBack)
			m_pCallBack(JOURNAL_MESSAGE, (LPARAM)"Execute State : PLAY_START");

		if(LoadJournal(m_nRunningIndex)) {
			m_nRepeatCount = 0;
			m_nScenarioState = PLAY_NEXT;
			InstallFilter(JOURNAL_PLAYBACK_INDEX, TRUE);

			if(	GetAt(m_nRunningIndex).m_fCheckStop &&
				CTime::GetCurrentTime() < GetAt(m_nRunningIndex).m_tStopTime + m_tsEverydayAdjustTime) {
				m_uMainTimerID = SetTimer(NULL, NULL,
					1000 * (GetAt(m_nRunningIndex).m_tStopTime + m_tsEverydayAdjustTime - CTime::GetCurrentTime()).GetTotalSeconds(),
					CJournalRecorder::StopTimerProc);
				TRACE("\tWait %d seconds\n", (GetAt(m_nRunningIndex).m_tStopTime + m_tsEverydayAdjustTime - CTime::GetCurrentTime()).GetTotalSeconds());
			}
			if(	GetAt(m_nRunningIndex).m_fCheckDuration) {
				m_tDurationStopTime = CTime::GetCurrentTime() + GetAt(m_nRunningIndex).m_tsDurationTime;
				m_uDurationTimerID = SetTimer(NULL, NULL,
					1000 * GetAt(m_nRunningIndex).m_tsDurationTime.GetTotalSeconds(),
					CJournalRecorder::DurationTimerProc);
				TRACE("\tPlay during %d seconds\n", GetAt(m_nRunningIndex).m_tsDurationTime.GetTotalSeconds());
			}
		} else {
			if(	GetAt(m_nRunningIndex).m_fCheckStop &&
				GetAt(m_nRunningIndex).m_tStopTime + m_tsEverydayAdjustTime > CTime::GetCurrentTime()) {
				m_nScenarioState = PLAY_NEXT;
				m_uMainTimerID = SetTimer(NULL, NULL,
					1000 * (GetAt(m_nRunningIndex).m_tStopTime + m_tsEverydayAdjustTime - CTime::GetCurrentTime()).GetTotalSeconds(),
					CJournalRecorder::StopTimerProc);
				TRACE("\tWait %d seconds\n", (GetAt(m_nRunningIndex).m_tStopTime + m_tsEverydayAdjustTime - CTime::GetCurrentTime()).GetTotalSeconds());
			} else {
				m_nScenarioState = STOP;
				NextScenarioState();
			}
		}

		break;
	case PLAY_NEXT:
		TRACE("Execute State : PLAY_NEXT\n");
//		if(m_pCallBack)
//			m_pCallBack(JOURNALMESSAGE, (LPARAM)"Execute State : PLAY_NEXT");

		break;
	case PLAY_STOP:
		TRACE("Execute State : PLAY_STOP\n");
		if(m_pCallBack)
			m_pCallBack(JOURNAL_MESSAGE, (LPARAM)"Execute State : PLAY_STOP");

		if(GetAt(m_nRunningIndex).m_fCheckDuration) {
			if(CTime::GetCurrentTime() < m_tDurationStopTime) {
				m_nScenarioState = PLAY_NEXT;
				InstallFilter(JOURNAL_PLAYBACK_INDEX, TRUE);
			} else {
				m_nScenarioState = WAIT_STOP;
				NextScenarioState();
			}
		} else if(GetAt(m_nRunningIndex).m_fCheckRepeat) {
		
			if(!GetAt(m_nRunningIndex).m_nEditRepeat) {
				m_nScenarioState = PLAY_NEXT;
				InstallFilter(JOURNAL_PLAYBACK_INDEX, TRUE);
			} else {
				m_nRepeatCount++;

				TRACE("\tRepeat %d\n", m_nRepeatCount);

				if(m_nRepeatCount < GetAt(m_nRunningIndex).m_nEditRepeat) {
					m_nScenarioState = PLAY_NEXT;
					InstallFilter(JOURNAL_PLAYBACK_INDEX, TRUE);
				} else if(!GetAt(m_nRunningIndex).m_fCheckStop) {
					m_nScenarioState = STOP;
					NextScenarioState();
				}
			}
		} else {
			if(!GetAt(m_nRunningIndex).m_fCheckStop) {
				m_nScenarioState = STOP;
				NextScenarioState();
			}
		}

		break;
	case WAIT_STOP:
		TRACE("Execute State : WAIT_STOP\n");
		if(m_pCallBack)
			m_pCallBack(JOURNAL_MESSAGE, (LPARAM)"Execute State : WAIT_STOP");

		if(GetAt(m_nRunningIndex).m_tStopTime + m_tsEverydayAdjustTime <= CTime::GetCurrentTime()) {
			m_nScenarioState = STOP;
			NextScenarioState();
		} else
			m_uMainTimerID = SetTimer(NULL, NULL, 1000, CJournalRecorder::StopTimerProc);

		break;
	case STOP:
		TRACE("Execute State : STOP\n");
		if(m_pCallBack)
			m_pCallBack(JOURNAL_MESSAGE, (LPARAM)"Execute State : STOP");


		if(GetAt(m_nRunningIndex).m_fCheckDuration)
			KillTimer(NULL, AfxGetJournalRecorder()->m_uDurationTimerID);

		InstallFilter(JOURNAL_PLAYBACK_INDEX, FALSE);

		if(m_fUseItemSelected)
			while(++m_nRunningIndex < m_anItemSelected.GetSize() && !m_anItemSelected[m_nRunningIndex]);
		else
			++m_nRunningIndex;

		if(m_nRunningIndex < GetSize()) {
			m_nScenarioState = IDLE;
			NextScenarioState();
		} else { // Alors c'est la fin du scenario
			Stop();
			if(m_pCallBack)
				m_pCallBack(SCENARIO_STOP_PLAYING, (LPARAM)NULL);
		}

		break;
	}

	return TRUE;
}

VOID CALLBACK CJournalRecorder::StartTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
	KillTimer(NULL, AfxGetJournalRecorder()->m_uMainTimerID);
	AfxGetJournalRecorder()->m_nScenarioState = WAIT_START;
	AfxGetJournalRecorder()->NextScenarioState();
}

VOID CALLBACK CJournalRecorder::DurationTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
	KillTimer(NULL, AfxGetJournalRecorder()->m_uDurationTimerID);
	AfxGetJournalRecorder()->m_nScenarioState = WAIT_STOP;
	AfxGetJournalRecorder()->NextScenarioState();
}

VOID CALLBACK CJournalRecorder::StopTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
	KillTimer(NULL, AfxGetJournalRecorder()->m_uMainTimerID);
	AfxGetJournalRecorder()->m_nScenarioState = WAIT_STOP;
	AfxGetJournalRecorder()->NextScenarioState();
}

//---------------------------------------------------------------------------
// InstallSysMsgFilter
//
// Install / Remove Filter function for the WH_SYSMSGFILTER
//
//---------------------------------------------------------------------------
int CJournalRecorder::InstallFilter(int nHookIndex, int nCode )
{
	if(!m_fInitCalled || nHookIndex < 0)
		return -1;

	if(m_pCallBack)
		m_pCallBack(nHookIndex, nCode);

	if(nCode) {
		m_hhookHooks[nHookIndex] =
			SetWindowsHookEx(m_nHookCodes[nHookIndex], (HOOKPROC)m_lpfnHookProcs[nHookIndex], AfxGetInstanceHandle(), 0);
		m_nHookStates[nHookIndex] = TRUE;
		if(nHookIndex == JOURNAL_PLAYBACK_INDEX)
			m_lpPlayEvent = NULL;
	} else if(m_nHookStates[nHookIndex]) {
		UnhookWindowsHookEx(m_hhookHooks[nHookIndex]);
		m_nHookStates[nHookIndex] = FALSE;
	}

	return 0;
}

//---------------------------------------------------------------------------
// JournalPlaybackFunc
//
// Filter function for the WH_JOURNALPLAYBACK
//
//---------------------------------------------------------------------------
LRESULT CALLBACK CJournalRecorder::JournalPlaybackFunc (int nCode, WPARAM wParam, LPARAM lParam )
{
	static	  int		 nRepeatRequests;
	static	  DWORD		 dwTimeAdjust;
	static	  DWORD		 dwLastEventTime;
			  LPEVENTMSG lpEvent;
			  long		 lReturnValue;
	CJournalRecorder * jr = AfxGetJournalRecorder();

	if ( nCode >= 0 ) {
		// No Playback if we haven't recorded an Event
		//
		// No Playback While recording.
		// This is not a limitation of the hooks.
		// This is only because of the simple event storage used in this example
		//
		// We should never get here since the enable / disable menu stuff should
		// make getting here impossible
		//
		if(jr->m_lpEventChain == NULL || jr->m_nHookStates[JOURNAL_RECORD_INDEX]) {
			jr->InstallFilter(JOURNAL_PLAYBACK_INDEX, FALSE);
		

			if(jr->m_fIsRunning && jr->m_fPlayMode == MODE_SCENARIO) {
				jr->m_nScenarioState = PLAY_STOP;
				jr->NextScenarioState();
			}

			return((int)CallNextHookEx(jr->m_hhookHooks[JOURNAL_PLAYBACK_INDEX], nCode, wParam, lParam ));
		}

		if(jr->m_fIsRunning && jr->m_fPlayMode == MODE_SCENARIO)
			jr->NextScenarioState();

		if(jr->m_lpPlayEvent == NULL) {
			jr->m_lpPlayEvent = jr->m_lpEventChain;
//			lpLastEvent = NULL;	   // For the next time we start the recorder
			dwTimeAdjust = GetTickCount() - jr->m_dwStartRecordTime;
			dwLastEventTime = (DWORD) GetTickCount();
			nRepeatRequests = 1;
		}

		if(nCode == HC_SKIP) {
			nRepeatRequests = 1;

			if(jr->m_lpPlayEvent->lpNextEvent == NULL) {
				//			  free(lpEventChain);
				//			  lpEventChain = 
				jr->m_lpPlayEvent = NULL ;
				jr->InstallFilter(JOURNAL_PLAYBACK_INDEX, FALSE);

				if(jr->m_fIsRunning) {
					switch(jr->m_fPlayMode) {
					case MODE_SCENARIO:
						jr->m_nScenarioState = PLAY_STOP;
						jr->NextScenarioState();
						break;
					case MODE_JOURNAL:
						if(jr->m_fLoopPlay)
							jr->InstallFilter(JOURNAL_PLAYBACK_INDEX, TRUE);
						else // On arrete de jouer !
							jr->Stop();
						break;
					}
				}
			} else {
				dwLastEventTime = jr->m_lpPlayEvent->Event.time;
				jr->m_lpPlayEvent = jr->m_lpPlayEvent->lpNextEvent;
				//			  free(lpEventChain);
				//			  lpEventChain = lpPlayEvent;
			}
		} else if(nCode == HC_GETNEXT) {
			lpEvent = (LPEVENTMSG) lParam;
			lpEvent->message = jr->m_lpPlayEvent->Event.message;
			lpEvent->paramL  = jr->m_lpPlayEvent->Event.paramL;
			lpEvent->paramH  = jr->m_lpPlayEvent->Event.paramH;
			lpEvent->time    = jr->m_lpPlayEvent->Event.time + dwTimeAdjust;

			if(jr->m_fIsRunning && lpEvent->message == WM_MOUSEMOVE && jr->m_fShakeMouse) {
					short xPos = lpEvent->paramL;
					short yPos = lpEvent->paramH;
	
					xPos += (short)((double)jr->m_nShakeRange * (2.0 * (double)rand() / (double)RAND_MAX - 1.0));
					yPos += (short)((double)jr->m_nShakeRange * (2.0 * (double)rand() / (double)RAND_MAX - 1.0));
					lpEvent->paramL = xPos;
					lpEvent->paramH = yPos;
			}

			lReturnValue = lpEvent->time - GetTickCount();
			//
			// No Long ( negative ) waits
			//
			if ( lReturnValue < 0L ) {
				lReturnValue  = 0L;
				lpEvent->time = GetTickCount();
			}
			return ( (DWORD) lReturnValue );
		}
	}

	return CallNextHookEx(jr->m_hhookHooks[JOURNAL_PLAYBACK_INDEX], nCode, wParam, lParam);
}

//---------------------------------------------------------------------------
// JournalRecordFunc
//
// Filter function for the WH_JOURNALRECORD
//
//---------------------------------------------------------------------------
LRESULT CALLBACK CJournalRecorder::JournalRecordFunc (int nCode, WPARAM wParam, LPARAM lParam )
{
	SEventNode  *lpEventNode;
	LPEVENTMSG  lpEvent;
	CJournalRecorder * jr = AfxGetJournalRecorder();

	if ( nCode >= 0 ) {

		lpEvent = (LPEVENTMSG) lParam;

		//	TRACE("paramL = 0x%0x (%d), paramH = 0x%0x (%d)\n", lpEvent->paramL, lpEvent->paramL, lpEvent->paramH, lpEvent->paramH);
		//
		// Skip recording while playing back
		// This is not a limitation of the hooks.
		// This is only because of the simple event storage used in this example
		//
		if(jr->m_nHookStates[JOURNAL_PLAYBACK_INDEX])
			return 0;

		//
		// Stop recording ?
		//
		if(lpEvent->message == WM_KEYDOWN && LOBYTE(lpEvent->paramL)  == VK_F2 ) {
			jr->InstallFilter(JOURNAL_RECORD_INDEX, FALSE);
			if(jr->m_pCallBack)
				jr->m_pCallBack(JOURNAL_STOP_RECORDING, (LPARAM)NULL);
			return 0;
		}

		if((lpEventNode = (SEventNode *)malloc(sizeof(SEventNode))) == NULL)
			jr->InstallFilter(JOURNAL_RECORD_INDEX, FALSE);

		if(jr->m_lpLastEvent == NULL) {
			jr->m_dwStartRecordTime = (DWORD)GetTickCount();
			jr->m_lpEventChain = lpEventNode;
		} else
			jr->m_lpLastEvent->lpNextEvent = lpEventNode;

		jr->m_lpLastEvent = lpEventNode;
		jr->m_lpLastEvent->lpNextEvent = NULL;

		jr->m_lpLastEvent->Event.message = lpEvent->message;
		jr->m_lpLastEvent->Event.paramL  = lpEvent->paramL;
		jr->m_lpLastEvent->Event.paramH  = lpEvent->paramH;
		jr->m_lpLastEvent->Event.time	 = lpEvent->time;

		return 0;
	}

	return (CallNextHookEx(jr->m_hhookHooks[JOURNAL_RECORD_INDEX], nCode, wParam, lParam));
}

BOOL CJournalRecorder::SaveJournal(LPCTSTR pszFilename)
{
	CFile file;
		
	if(!file.Open(pszFilename, CFile::modeCreate | CFile::modeWrite | CFile::typeBinary))
		return FALSE;

	SEventNode * lpEventNode = m_lpEventChain, * lpNextEvent, sEventNode;

	while(lpEventNode) {
		lpNextEvent = lpEventNode->lpNextEvent;
		sEventNode = *lpEventNode;
		sEventNode.Event.time -= m_dwStartRecordTime;
		file.Write(&sEventNode, sizeof(SEventNode));
		lpEventNode = lpNextEvent;
	}

	file.Close();

	return TRUE;
}

BOOL CJournalRecorder::LoadJournal(int nIndex)
{
	if(m_pCallBack)
		m_pCallBack(JOURNAL_LOAD, (LPARAM)nIndex);

	return nIndex >= 0 && nIndex < GetSize() ? LoadJournal(GetAt(nIndex).m_szEditFile) : FALSE;
}

BOOL CJournalRecorder::LoadJournal(LPCTSTR pszFilename)
{
	CFile file;
	
	::SetCurrentDirectory(AfxGetAppPath());

	if(!file.Open(pszFilename, CFile::modeRead | CFile::typeBinary))
		return FALSE;

	SEventNode * lpEventNode = NULL, sEventNode;
	EraseJournal();
	m_dwStartRecordTime = 0;

	while(file.Read(&sEventNode, sizeof(SEventNode)) == sizeof(SEventNode)) {

		if((lpEventNode = (SEventNode *)malloc(sizeof(SEventNode))) == NULL ) {
			file.Close();
			return FALSE;
		}

		*lpEventNode = sEventNode;

		if(m_lpLastEvent == NULL )
			m_lpEventChain = lpEventNode;
		else
			m_lpLastEvent->lpNextEvent = lpEventNode;

		m_lpLastEvent = lpEventNode;
		m_lpLastEvent->lpNextEvent = NULL;
	}

	file.Close();

	return TRUE;
}

DWORD CJournalRecorder::GetJournalTime(LPCTSTR pszFilename)
{
	CFile file;
		
	if(!file.Open(pszFilename, CFile::modeRead | CFile::typeBinary))
		return 0;

	SEventNode sEventNode;
	DWORD dwRecordTime = 0;

	while(file.Read(&sEventNode, sizeof(SEventNode)) == sizeof(SEventNode)) {
		dwRecordTime = sEventNode.Event.time;
		if(!sEventNode.lpNextEvent)
			break;
	}

	file.Close();
	return dwRecordTime;
}

BOOL CJournalRecorder::EraseJournal()
{
	InstallFilter(JOURNAL_RECORD_INDEX, FALSE);
	InstallFilter(JOURNAL_PLAYBACK_INDEX, FALSE);

	SEventNode * lpEventNode = m_lpEventChain;

	while(lpEventNode) {
		m_lpLastEvent = lpEventNode->lpNextEvent;
		free(lpEventNode);
		lpEventNode = m_lpLastEvent;
	}

	m_lpEventChain = NULL;	// Head of recorded Event List
	m_lpLastEvent	= NULL;		// Tail of recorded Event List
	m_lpPlayEvent	= NULL;		// Current Event being played

	return TRUE;
}

BOOL CJournalRecorder::SaveScenario(LPCTSTR pszFilename)
{
	CFile file;
		
	if(!file.Open(pszFilename, CFile::modeCreate | CFile::modeWrite | CFile::typeBinary))
		return FALSE;

	CArchive ar(&file, CArchive::store);
	
	int nItemCount = GetSize();

	ar << m_fPlayEveryday;
	ar << nItemCount;

	for(int iItem = 0; iItem < nItemCount; iItem++)
		ar << GetAt(iItem);

	ar.Close();
	file.Close();

	return TRUE;
}

BOOL CJournalRecorder::MergeScenario(LPCTSTR pszFilename)
{
	CFile file;
		
	if(!file.Open(pszFilename, CFile::modeRead | CFile::typeBinary))
		return FALSE;

	CArchive ar(&file, CArchive::load);

	CJournalAction a;
	int nItemCount = 0;

	BOOL fPlayEveryday;

	ar >> fPlayEveryday;
	ar >> nItemCount;

	for(int iItem = 0; iItem < nItemCount; iItem++) {
		ar >> a;
		Add(a);
	}

	ar.Close();
	file.Close();

	return TRUE;
}

BOOL CJournalRecorder::LoadScenario(LPCTSTR pszFilename)
{
	CFile file;
		
	if(!file.Open(pszFilename, CFile::modeRead | CFile::typeBinary))
		return FALSE;

	CArchive ar(&file, CArchive::load);

	RemoveAll();

	CJournalAction a;
	int nItemCount = 0;

	ar >> m_fPlayEveryday;
	ar >> nItemCount;

	for(int iItem = 0; iItem < nItemCount; iItem++) {
		ar >> a;
		Add(a);
	}

	ar.Close();
	file.Close();

	return TRUE;
}

CJournalAction & CJournalAction::operator =(const CJournalAction &a)
{
	m_szEditFile = a.m_szEditFile;
	m_fCheckStart = a.m_fCheckStart;
	m_tStartTime = a.m_tStartTime;
	m_fCheckRepeat = a.m_fCheckRepeat;
	m_nEditRepeat = a.m_nEditRepeat;
	m_fCheckDuration = a.m_fCheckDuration;
	m_tsDurationTime = a.m_tsDurationTime;
	m_fCheckStop = a.m_fCheckStop;
	m_tStopTime = a.m_tStopTime;
	return *this;
}

CArchive& AFXAPI operator<<(CArchive& ar, const CJournalAction& a)
{
	ar << a.m_szEditFile;
	ar << a.m_fCheckStart;
	ar << a.m_tStartTime;
	ar << a.m_fCheckRepeat;
	ar << a.m_nEditRepeat;
	ar << a.m_fCheckDuration;
	ar << a.m_tsDurationTime;
	ar << a.m_fCheckStop;
	ar << a.m_tStopTime;
	return ar;
}

CArchive& AFXAPI operator>>(CArchive& ar, CJournalAction& a)
{
	ar >> a.m_szEditFile;
	ar >> a.m_fCheckStart;
	ar >> a.m_tStartTime;
	ar >> a.m_fCheckRepeat;
	ar >> a.m_nEditRepeat;
	ar >> a.m_fCheckDuration;
	ar >> a.m_tsDurationTime;
	ar >> a.m_fCheckStop;
	ar >> a.m_tStopTime;
	return ar;
}

void CJournalRecorder::EnableScreenSaver(BOOL fEnable)
{
	if(m_fScreenSaverWasActived)
		SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, fEnable, 0, SPIF_SENDWININICHANGE);
}

void CJournalRecorder::Stop()
{
	TRACE("CJournalRecorder::Stop\n");

	InstallFilter(JOURNAL_PLAYBACK_INDEX, FALSE);

	EnableScreenSaver();
	m_fIsRunning = FALSE;

	if(m_pCallBack)
		m_pCallBack(JOURNAL_MESSAGE, (LPARAM)"The End");
}

BOOL CJournalRecorder::IsRecording()
{
	return m_nHookStates[JOURNAL_RECORD_INDEX];
}

BOOL CJournalRecorder::IsPlaying()
{
	return m_nHookStates[JOURNAL_PLAYBACK_INDEX];
}

BOOL CJournalRecorder::IsEmpty()
{
	return m_lpEventChain == NULL;
}
