// JournalRecorder.h: interface for the CJournalRecorder class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_JOURNALRECORDER_H_INCLUDED_)
#define AFX_JOURNALRECORDER_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define NUMOFHOOKS 2
#define JOURNAL_PLAYBACK_INDEX 0
#define JOURNAL_RECORD_INDEX 1

#define JOURNAL_MESSAGE 3
#define JOURNAL_LOAD 4
#define JOURNAL_STOP_RECORDING 5
#define SCENARIO_STOP_PLAYING 6

typedef struct TAGEventNode {
	EVENTMSG	Event;
	struct TAGEventNode * lpNextEvent;
} SEventNode;

class CJournalAction : public CObject
{
public:
	CJournalAction() {}
	CJournalAction(const CJournalAction & a) { *this = a; }

	CJournalAction & operator =(const CJournalAction & a);
	friend CArchive& AFXAPI operator<<(CArchive& ar, const CJournalAction& a);
	friend CArchive& AFXAPI operator>>(CArchive& ar, CJournalAction& a);

	CString m_szEditFile;

	BOOL m_fCheckStart;
	CTime m_tStartTime;

	BOOL m_fCheckRepeat;
	int m_nEditRepeat;

	BOOL m_fCheckDuration;
	CTimeSpan m_tsDurationTime;

	BOOL m_fCheckStop;
	CTime m_tStopTime;
};

typedef enum EScenarioState { IDLE = 0, WAIT_START, PLAY_START, PLAY_NEXT, PLAY_STOP, WAIT_STOP, STOP};

typedef enum EPlayMode { MODE_IDLE = FALSE, MODE_JOURNAL, MODE_SCENARIO };

class CJournalRecorder : public CArray<CJournalAction, CJournalAction&>
{
public:
	BOOL IsEmpty();
	BOOL IsPlaying();
	BOOL IsRecording();
	void Stop();
	void EnableScreenSaver(BOOL fEnable = TRUE);
	CJournalRecorder();
	virtual ~CJournalRecorder();

	BOOL Init(void (* pCallBack)(int, LPARAM) = NULL);

	int InstallFilter (int nHookIndex, int nCode);
	BOOL SaveJournal(LPCTSTR pszFilename);
	BOOL LoadJournal(LPCTSTR pszFilename);
	BOOL LoadJournal(int nIndex);
	BOOL EraseJournal();
	DWORD GetJournalTime(LPCTSTR pszFilename);

	BOOL SaveScenario(LPCTSTR pszFilename);
	BOOL MergeScenario(LPCTSTR pszFilename);
	BOOL LoadScenario(LPCTSTR pszFilename);
	BOOL RunScenario();
	BOOL RunScenario(CByteArray & anItemSelected);
	BOOL NextScenarioState();
	UINT m_uMainTimerID;
	UINT m_uDurationTimerID;

	BOOL m_fIsRunning;

	BOOL m_fPlayEveryday;

	EPlayMode m_fPlayMode;

	BOOL m_fLoopPlay;

	BOOL m_fShakeMouse;
	int m_nShakeRange;

protected:

	// Le callback de la fenetre principale appelee par cet objet
	// lorsqu'un evenement survient...
	void (* m_pCallBack)(int nType, LPARAM lParam);

	static LRESULT CALLBACK JournalPlaybackFunc (int nCode, WPARAM wParam, LPARAM lParam);
	static LRESULT CALLBACK JournalRecordFunc (int nCode, WPARAM wParam, LPARAM lParam);
	static VOID CALLBACK StartTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime);
	static VOID CALLBACK DurationTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime);
	static VOID CALLBACK StopTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime);

	BOOL m_fScreenSaverWasActived;

	BOOL m_fUseItemSelected;
	CByteArray m_anItemSelected;

	int m_nRunningIndex;
	EScenarioState m_nScenarioState;
	int m_nRepeatCount;
	CTimeSpan m_tsEverydayAdjustTime;
	CTime m_tDurationStopTime;

	BOOL m_fInitCalled;				// Has the Init function been called ?
	DWORD m_dwStartRecordTime;		// Time JournalRecord Started

	SEventNode * m_lpEventChain;  // Head of recorded Event List
	SEventNode * m_lpLastEvent;  // Tail of recorded Event List
	SEventNode * m_lpPlayEvent;  // Current Event being played
	//  My Hook States
	int m_nHookStates[NUMOFHOOKS]; // State Table of my hooks
	//  Hook Codes
	int m_nHookCodes[NUMOFHOOKS];
	// Filter Function Addresses
	FARPROC m_lpfnHookProcs[NUMOFHOOKS];
	// Hook Handles
	HHOOK m_hhookHooks[NUMOFHOOKS];
};

#endif // !defined(AFX_JOURNALRECORDER_H_INCLUDED_)
